;;; ox-bibtex.el --- Export bibtex fragments

;; Copyright (C) 2009-2013 Taru Karttunen

;; Author: Taru Karttunen <taruti@taruti.net>

;; This file is not currently part of GNU Emacs.

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 2, or (at
;; your option) any later version.

;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program ; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; This is an utility to handle BibTeX export to both LaTeX and html
;; exports. It uses the bibtex2html software from
;; http://www.lri.fr/~filliatr/bibtex2html/
;;
;; The usage is as follows:
;; #+BIBLIOGRAPHY: bibfilebasename stylename optional-options
;; e.g. given foo.bib and using style plain:
;; #+BIBLIOGRAPHY: foo plain option:-d
;;
;; Optional options are of the form:
;;
;; option:-foobar pass '-foobar' to bibtex2html
;; e.g.
;; option:-d sort by date.
;; option:-a sort as BibTeX (usually by author) *default*
;; option:-u unsorted i.e. same order as in .bib file
;; option:-r reverse the sort.
;; see the bibtex2html man page for more. Multiple options can be combined like:
;; option:-d option:-r
;;
;; Limiting to only the entries cited in the document:
;; limit:t

;; For LaTeX export this simply inserts the lines
;; \bibliographystyle{plain}
;; \bibliography{foo}
;; into the tex-file when exporting.

;; For HTML export it:
;; 1) converts all \cite{foo} to links to the bibliography
;; 2) creates a foo.html and foo_bib.html
;; 3) includes the contents of foo.html in the exported html file

(require 'ox)

(defun org-export-bibtex-preprocess (backend)
  "Export all BibTeX."
  (interactive)
  (let ((cite-alist '())
        (map-cites
         (lambda (f)
           ;; Call function F with point on each "\cite{...}"
           ;; construct in the current buffer.  Function F is called
           ;; with one argument: the contents of the "\cite" macro.
           (save-excursion
             (save-match-data
               (goto-char (point-min))
               (while (re-search-forward "\\\\cite{\\([^}\n]+\\)}" nil t)
                 (when (eq (org-element-type
                            (save-match-data (org-element-context)))
                           'latex-fragment)
                   (funcall f (match-string 1)))))))))
    ;; Convert #+BIBLIOGRAPHY: name style.
    (goto-char (point-min))
    (while (re-search-forward "^[ \t]*#\\+BIBLIOGRAPHY:" nil t)
      (let* ((element (org-element-at-point))
             (value (org-element-property :value element)))
        (when (and (eq (org-element-type element) 'keyword)
                   value
                   (string-match "\\(\\S-+\\)[ \t]+\\(\\S-+\\)\\(.*\\)"
                                 value))
          (let ((ind (make-string (org-get-indentation) ?\s))
                (file (match-string 1 value))
                (style (match-string 2 value))
                (opt (mapcar
                      (lambda (o)
                        (let ((s (split-string o ":")))
                          (cons (car s) (nth 1 s))))
                      (split-string (match-string 3 value) nil t))))
            ;; Remove keyword.
            (delete-region
             (org-element-property :begin element)
             (progn (goto-char (org-element-property :end element))
                    (skip-chars-backward " \r\t\n")
                    (line-end-position)))
            (insert
             (case backend
               ;; LaTeX export.
               (latex
                (concat ind "#+LATEX: \\bibliographystyle{" style "}\n"
                        ind "#+LATEX: \\bibliography{" file "}"))
               ;; HTML export.
               (html
                (let (extra-args cite-list end-hook tmp-files)
                  (dolist (elt opt)
                    (when (equal "option" (car elt))
                      (push (cdr elt) extra-args)))
                  (when (assoc "limit" opt)
                    ;; Limit is true: collect references.
                    (funcall map-cites
                             (lambda (cite)
                               (dolist (c (org-split-string cite ","))
                                 (add-to-list 'cite-list c))))
                    (let ((tmp (make-temp-file "org-exp-bibtex")))
                      (with-temp-file tmp
                        (dolist (i cite-list) (insert (concat i "\n"))))
                      (push tmp tmp-files)
                      (setq extra-args
                            (append extra-args `("-citefile" ,tmp)))))
                  (save-window-excursion
                    (unless (eq 0 (apply 'call-process
                                         (append '("bibtex2html" nil nil nil)
                                                 `("-a" "--nodoc"  "--style" ,style "--no-header")
                                                 extra-args
                                                 (list (concat file ".bib")))))
                      (error "Executing bibtex2html failed")))
                  (dolist (f tmp-files) (delete-file f)))
                (with-temp-buffer
                  (save-match-data
                    (insert-file-contents (concat file ".html"))
                    (goto-char (point-min))
                    (let ((link-re (org-re "a name=\"\\([-_[:word:]]+\\)\">\\([[:word:]]+\\)")))
                      (while (re-search-forward link-re nil t)
                        (push (cons (match-string 1) (match-string 2))
                              cite-alist)))
                    (goto-char (point-min))
                    (while (search-forward "<hr>" nil t)
                      (replace-match "<hr/>" t t))
                    (concat ind "#+BEGIN_HTML\n"
                            "<div id=\"bibliography\">\n<h2>References</h2>\n"
                            (buffer-string) "\n"
                            "</div>\n"
                            ind "#+END_HTML"))))))))))
    ;; Convert cites to links in HTML.
    (when (eq backend 'html)
      ;; Split citation commands with multiple keys.
      (funcall
       map-cites
       (lambda (cite)
         (let ((keys (save-match-data (org-split-string cite ","))))
           (when (cdr keys)
             (replace-match
              (mapconcat (lambda (k) (format "\\cite{%s}" k)) keys "")
              t t)))))
      ;; Replace the citation commands with links.
      (funcall
       map-cites
       (lambda (cite)
         (let ((cv (assoc cite cite-alist)))
           (replace-match (format "[[#%s][%s]]" cite (if cv (cdr cv) cite))
                          t t)))))))


(add-hook 'org-export-before-processing-hook 'org-export-bibtex-preprocess)


(provide 'ox-bibtex)

;;; ox-bibtex.el ends here
